require_relative 'firestore-odm/document'
require_relative 'firestore-odm/model'

class Google::Cloud::Firestore::DocumentReference
  def to_s
    document_id
  end
end

class Module
  # Defines a setter method.
  # @param name [String, Symbol]
  # @param block
  def define_setter_method name, &block
    define_method("#{name}=", &block)
  end
end

module FirestoreODM
  attr_reader :conversions, :options

  @conversions = {
    String => :to_s,
    Integer => :to_i,
    Float => :to_f,
    Date => :to_date,
    DateTime => :to_datetime,
    Time => :to_time
  }

  @options = {}

  # Sets a conversion rule for a certain type.
  # @param type [Class]
  # @param target [Symbol, Proc]
  def add_conversion type, target
    @conversions[type] = target
    return
  end

  # Gets a conversion.
  # @param type [Class]
  # @return [Proc]
  def get_conversion type
    case rule = conversions[type]
    when Symbol
      Proc.new { |value| value.method(rule).call }
    when Proc
      rule
    else
      raise "Conversion not found for type `#{type}`"
    end
  end

  # @param name [Symbol]
  def add_option name, &block
    @options[name] = block
  end

  def get_option name, option
    method = @options[name]

    case option
    when Proc
      get_option name, option.call
    else
      method.call option
    end
  end

  extend self
end


FirestoreODM.add_option :more_than do |value|
  Proc.new { |o| value < o }
end

FirestoreODM.add_option :min do |value|
  Proc.new { |o| value <= o }
end

FirestoreODM.add_option :max do |value|
  Proc.new { |o| o <= value }
end

FirestoreODM.add_option :less_than do |value|
  Proc.new { |o| o < value }
end

FirestoreODM.add_option :in do |list|
  Proc.new { |o| list.include? o }
end