require 'google/cloud/firestore'

module FirestoreODM
  module Collection
    # Returns the collection used by the model.
    # @param root [Google::Cloud::Firestore::DocumentReference]
    # @return [Google::Cloud::Firestore::CollectionReference] the collection.
    def collection root = nil
      (root or Google::Cloud::Firestore.new).collection @path
    end

    # Sets the collection name.
    # @param name [String, Symbol] the collection name.
    def path name
      @path = name
      return
    end

    # Creates a document.
    # @param opts [Hash] a hash with options.
    # @yield [doc] 
    # @return [FirestoreODM::Document] the document.
    def create opts = {}, &block
      block.call o = self.new

      doc = collection(opts[:relative_to]).add o.changes
      self.new doc
    end

    # Creates a document with an specific id.
    # @param id [String] the document id.
    # @param opts [Hash]
    # @yield [doc] 
    # @return [FirestoreODM::Document] the document.
    def create_with_id id, opts = {}, &block
      block.call o = self.new

      doc = collection(opts[:relative_to]).doc id
      doc.create o.changes
      self.from_id id, opts
    end

    # Gets a document.
    # @param id [String] 
    # @param opts [Hash]
    # @return [FirestoreODM::Document] the document.
    def from_id id, opts = {}
      doc = collection(opts[:relative_to]).doc id
      self.new doc
    end
  end
end